# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=useless-super-delegation

import datetime
from typing import Any, Mapping, Optional, TYPE_CHECKING, Union, overload

from .._utils.model_base import Model as _Model, rest_field

if TYPE_CHECKING:
    from .. import models as _models


class EkmConnection(_Model):
    """A EkmConnection model object.

    :ivar host: EKM proxy FQDN (Fully Qualified Domain Name). Only allowed characters are a-z, A-Z,
     0-9, hyphen (-), dot (.), and colon (:). Required.
    :vartype host: str
    :ivar path_prefix: Optional path prefix for the EKM proxy (if any).
    :vartype path_prefix: str
    :ivar server_ca_certificates: The root CA certificate chain that issued the proxy server's
     certificate. An array of certificates in the certificate chain, each in DER format and base64
     encoded. Required.
    :vartype server_ca_certificates: list[bytes]
    :ivar server_subject_common_name: The subject common name of the server certificate of EKM
     Proxy.
    :vartype server_subject_common_name: str
    """

    host: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """EKM proxy FQDN (Fully Qualified Domain Name). Only allowed characters are a-z, A-Z, 0-9, hyphen
     (-), dot (.), and colon (:). Required."""
    path_prefix: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Optional path prefix for the EKM proxy (if any)."""
    server_ca_certificates: list[bytes] = rest_field(
        visibility=["read", "create", "update", "delete", "query"], format="base64"
    )
    """The root CA certificate chain that issued the proxy server's certificate. An array of
     certificates in the certificate chain, each in DER format and base64 encoded. Required."""
    server_subject_common_name: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The subject common name of the server certificate of EKM Proxy."""

    @overload
    def __init__(
        self,
        *,
        host: str,
        server_ca_certificates: list[bytes],
        path_prefix: Optional[str] = None,
        server_subject_common_name: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class EkmProxyClientCertificateInfo(_Model):
    """EKM proxy client certificate information.

    :ivar ca_certificates: The client root CA certificate chain to authenticate to the EKM proxy.
     An array of certificates in the certificate chain, each in DER format and base64 encoded.
     Required.
    :vartype ca_certificates: list[bytes]
    :ivar subject_common_name: The subject common name of the client certificate used to
     authenticate to the EKM proxy. Required.
    :vartype subject_common_name: str
    """

    ca_certificates: list[bytes] = rest_field(visibility=["read"], format="base64")
    """The client root CA certificate chain to authenticate to the EKM proxy. An array of certificates
     in the certificate chain, each in DER format and base64 encoded. Required."""
    subject_common_name: str = rest_field(visibility=["read"])
    """The subject common name of the client certificate used to authenticate to the EKM proxy.
     Required."""


class EkmProxyInfo(_Model):
    """EKM proxy information.

    :ivar api_version: The highest version of proxy interface API supported by the EKM Proxy.
     Required.
    :vartype api_version: str
    :ivar proxy_vendor: The name of the proxy vendor. Required.
    :vartype proxy_vendor: str
    :ivar proxy_name: The name of the proxy product and its version. Required.
    :vartype proxy_name: str
    :ivar ekm_vendor: The name of the EKM vendor. Required.
    :vartype ekm_vendor: str
    :ivar ekm_product: The name of the EKM product and its version. Required.
    :vartype ekm_product: str
    """

    api_version: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The highest version of proxy interface API supported by the EKM Proxy. Required."""
    proxy_vendor: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The name of the proxy vendor. Required."""
    proxy_name: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The name of the proxy product and its version. Required."""
    ekm_vendor: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The name of the EKM vendor. Required."""
    ekm_product: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The name of the EKM product and its version. Required."""

    @overload
    def __init__(
        self,
        *,
        api_version: str,
        proxy_vendor: str,
        proxy_name: str,
        ekm_vendor: str,
        ekm_product: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class FullBackupOperation(_Model):
    """Full backup operation.

    :ivar status: Status of the backup operation. Known values are: "InProgress", "Succeeded",
     "Canceled", and "Failed".
    :vartype status: str or ~azure.keyvault.administration._generated.models.OperationStatus
    :ivar status_details: The status details of backup operation.
    :vartype status_details: str
    :ivar error: Error encountered, if any, during the full backup operation.
    :vartype error: ~azure.keyvault.administration._generated.models.FullBackupOperationError
    :ivar start_time: The start time of the backup operation in UTC.
    :vartype start_time: ~datetime.datetime
    :ivar end_time: The end time of the backup operation in UTC.
    :vartype end_time: ~datetime.datetime
    :ivar job_id: Identifier for the full backup operation.
    :vartype job_id: str
    :ivar azure_storage_blob_container_uri: The Azure blob storage container Uri which contains the
     full backup.
    :vartype azure_storage_blob_container_uri: str
    """

    status: Optional[Union[str, "_models.OperationStatus"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Status of the backup operation. Known values are: \"InProgress\", \"Succeeded\", \"Canceled\",
     and \"Failed\"."""
    status_details: Optional[str] = rest_field(
        name="statusDetails", visibility=["read", "create", "update", "delete", "query"]
    )
    """The status details of backup operation."""
    error: Optional["_models.FullBackupOperationError"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Error encountered, if any, during the full backup operation."""
    start_time: Optional[datetime.datetime] = rest_field(
        name="startTime", visibility=["read", "create", "update", "delete", "query"], format="unix-timestamp"
    )
    """The start time of the backup operation in UTC."""
    end_time: Optional[datetime.datetime] = rest_field(
        name="endTime", visibility=["read", "create", "update", "delete", "query"], format="unix-timestamp"
    )
    """The end time of the backup operation in UTC."""
    job_id: Optional[str] = rest_field(name="jobId", visibility=["read", "create", "update", "delete", "query"])
    """Identifier for the full backup operation."""
    azure_storage_blob_container_uri: Optional[str] = rest_field(
        name="azureStorageBlobContainerUri", visibility=["read", "create", "update", "delete", "query"]
    )
    """The Azure blob storage container Uri which contains the full backup."""

    @overload
    def __init__(
        self,
        *,
        status: Optional[Union[str, "_models.OperationStatus"]] = None,
        status_details: Optional[str] = None,
        error: Optional["_models.FullBackupOperationError"] = None,
        start_time: Optional[datetime.datetime] = None,
        end_time: Optional[datetime.datetime] = None,
        job_id: Optional[str] = None,
        azure_storage_blob_container_uri: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class FullBackupOperationError(_Model):
    """FullBackupOperationError.

    :ivar code: The error code.
    :vartype code: str
    :ivar message: The error message.
    :vartype message: str
    :ivar inner_error: The key vault server error.
    :vartype inner_error: ~azure.keyvault.administration._generated.models.FullBackupOperationError
    """

    code: Optional[str] = rest_field(visibility=["read"])
    """The error code."""
    message: Optional[str] = rest_field(visibility=["read"])
    """The error message."""
    inner_error: Optional["_models.FullBackupOperationError"] = rest_field(name="innererror", visibility=["read"])
    """The key vault server error."""


class KeyVaultError(_Model):
    """The key vault error exception.

    :ivar error: The key vault server error.
    :vartype error: ~azure.keyvault.administration._generated.models.FullBackupOperationError
    """

    error: Optional["_models.FullBackupOperationError"] = rest_field(visibility=["read"])
    """The key vault server error."""


class Permission(_Model):
    """Role definition permissions.

    :ivar actions: Action permissions that are granted.
    :vartype actions: list[str]
    :ivar not_actions: Action permissions that are excluded but not denied. They may be granted by
     other role definitions assigned to a principal.
    :vartype not_actions: list[str]
    :ivar data_actions: Data action permissions that are granted.
    :vartype data_actions: list[str or ~azure.keyvault.administration._generated.models.DataAction]
    :ivar not_data_actions: Data action permissions that are excluded but not denied. They may be
     granted by other role definitions assigned to a principal.
    :vartype not_data_actions: list[str or
     ~azure.keyvault.administration._generated.models.DataAction]
    """

    actions: Optional[list[str]] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Action permissions that are granted."""
    not_actions: Optional[list[str]] = rest_field(
        name="notActions", visibility=["read", "create", "update", "delete", "query"]
    )
    """Action permissions that are excluded but not denied. They may be granted by other role
     definitions assigned to a principal."""
    data_actions: Optional[list[Union[str, "_models.DataAction"]]] = rest_field(
        name="dataActions", visibility=["read", "create", "update", "delete", "query"]
    )
    """Data action permissions that are granted."""
    not_data_actions: Optional[list[Union[str, "_models.DataAction"]]] = rest_field(
        name="notDataActions", visibility=["read", "create", "update", "delete", "query"]
    )
    """Data action permissions that are excluded but not denied. They may be granted by other role
     definitions assigned to a principal."""

    @overload
    def __init__(
        self,
        *,
        actions: Optional[list[str]] = None,
        not_actions: Optional[list[str]] = None,
        data_actions: Optional[list[Union[str, "_models.DataAction"]]] = None,
        not_data_actions: Optional[list[Union[str, "_models.DataAction"]]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class PreBackupOperationParameters(_Model):
    """The authentication method and location for the backup operation.

    :ivar storage_resource_uri: Azure Blob storage container Uri.
    :vartype storage_resource_uri: str
    :ivar token: The SAS token pointing to an Azure Blob storage container.
    :vartype token: str
    :ivar use_managed_identity: Indicates which authentication method should be used. If set to
     true, Managed HSM will use the configured user-assigned managed identity to authenticate with
     Azure Storage. Otherwise, a SAS token has to be specified.
    :vartype use_managed_identity: bool
    """

    storage_resource_uri: Optional[str] = rest_field(
        name="storageResourceUri", visibility=["read", "create", "update", "delete", "query"]
    )
    """Azure Blob storage container Uri."""
    token: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The SAS token pointing to an Azure Blob storage container."""
    use_managed_identity: Optional[bool] = rest_field(
        name="useManagedIdentity", visibility=["read", "create", "update", "delete", "query"]
    )
    """Indicates which authentication method should be used. If set to true, Managed HSM will use the
     configured user-assigned managed identity to authenticate with Azure Storage. Otherwise, a SAS
     token has to be specified."""

    @overload
    def __init__(
        self,
        *,
        storage_resource_uri: Optional[str] = None,
        token: Optional[str] = None,
        use_managed_identity: Optional[bool] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class PreRestoreOperationParameters(_Model):
    """The authentication method and location for the restore operation.

    :ivar sas_token_parameters: A user-provided SAS token to an Azure blob storage container.
    :vartype sas_token_parameters:
     ~azure.keyvault.administration._generated.models.SASTokenParameter
    :ivar folder_to_restore: The Folder name of the blob where the previous successful full backup
     was stored.
    :vartype folder_to_restore: str
    """

    sas_token_parameters: Optional["_models.SASTokenParameter"] = rest_field(
        name="sasTokenParameters", visibility=["read", "create", "update", "delete", "query"]
    )
    """A user-provided SAS token to an Azure blob storage container."""
    folder_to_restore: Optional[str] = rest_field(
        name="folderToRestore", visibility=["read", "create", "update", "delete", "query"]
    )
    """The Folder name of the blob where the previous successful full backup was stored."""

    @overload
    def __init__(
        self,
        *,
        sas_token_parameters: Optional["_models.SASTokenParameter"] = None,
        folder_to_restore: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class RestoreOperation(_Model):
    """Restore operation.

    :ivar status: Status of the restore operation. Known values are: "InProgress", "Succeeded",
     "Canceled", and "Failed".
    :vartype status: str or ~azure.keyvault.administration._generated.models.OperationStatus
    :ivar status_details: The status details of restore operation.
    :vartype status_details: str
    :ivar error: Error encountered, if any, during the restore operation.
    :vartype error: ~azure.keyvault.administration._generated.models.FullBackupOperationError
    :ivar job_id: Identifier for the restore operation.
    :vartype job_id: str
    :ivar start_time: The start time of the restore operation.
    :vartype start_time: ~datetime.datetime
    :ivar end_time: The end time of the restore operation.
    :vartype end_time: ~datetime.datetime
    """

    status: Optional[Union[str, "_models.OperationStatus"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Status of the restore operation. Known values are: \"InProgress\", \"Succeeded\", \"Canceled\",
     and \"Failed\"."""
    status_details: Optional[str] = rest_field(
        name="statusDetails", visibility=["read", "create", "update", "delete", "query"]
    )
    """The status details of restore operation."""
    error: Optional["_models.FullBackupOperationError"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Error encountered, if any, during the restore operation."""
    job_id: Optional[str] = rest_field(name="jobId", visibility=["read", "create", "update", "delete", "query"])
    """Identifier for the restore operation."""
    start_time: Optional[datetime.datetime] = rest_field(
        name="startTime", visibility=["read", "create", "update", "delete", "query"], format="unix-timestamp"
    )
    """The start time of the restore operation."""
    end_time: Optional[datetime.datetime] = rest_field(
        name="endTime", visibility=["read", "create", "update", "delete", "query"], format="unix-timestamp"
    )
    """The end time of the restore operation."""

    @overload
    def __init__(
        self,
        *,
        status: Optional[Union[str, "_models.OperationStatus"]] = None,
        status_details: Optional[str] = None,
        error: Optional["_models.FullBackupOperationError"] = None,
        job_id: Optional[str] = None,
        start_time: Optional[datetime.datetime] = None,
        end_time: Optional[datetime.datetime] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class RestoreOperationParameters(_Model):
    """The authentication method and location for the restore operation.

    :ivar sas_token_parameters: A user-provided SAS token to an Azure blob storage container.
     Required.
    :vartype sas_token_parameters:
     ~azure.keyvault.administration._generated.models.SASTokenParameter
    :ivar folder_to_restore: The Folder name of the blob where the previous successful full backup
     was stored. Required.
    :vartype folder_to_restore: str
    """

    sas_token_parameters: "_models.SASTokenParameter" = rest_field(
        name="sasTokenParameters", visibility=["read", "create", "update", "delete", "query"]
    )
    """A user-provided SAS token to an Azure blob storage container. Required."""
    folder_to_restore: str = rest_field(
        name="folderToRestore", visibility=["read", "create", "update", "delete", "query"]
    )
    """The Folder name of the blob where the previous successful full backup was stored. Required."""

    @overload
    def __init__(
        self,
        *,
        sas_token_parameters: "_models.SASTokenParameter",
        folder_to_restore: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class RoleAssignment(_Model):
    """Role Assignments.

    :ivar id: The role assignment ID.
    :vartype id: str
    :ivar name: The role assignment name.
    :vartype name: str
    :ivar type: The role assignment type.
    :vartype type: str
    :ivar properties: Role assignment properties.
    :vartype properties:
     ~azure.keyvault.administration._generated.models.RoleAssignmentPropertiesWithScope
    """

    id: Optional[str] = rest_field(visibility=["read"])
    """The role assignment ID."""
    name: Optional[str] = rest_field(visibility=["read"])
    """The role assignment name."""
    type: Optional[str] = rest_field(visibility=["read"])
    """The role assignment type."""
    properties: Optional["_models.RoleAssignmentPropertiesWithScope"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Role assignment properties."""

    @overload
    def __init__(
        self,
        *,
        properties: Optional["_models.RoleAssignmentPropertiesWithScope"] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class RoleAssignmentCreateParameters(_Model):
    """Role assignment create parameters.

    :ivar properties: Role assignment properties. Required.
    :vartype properties: ~azure.keyvault.administration._generated.models.RoleAssignmentProperties
    """

    properties: "_models.RoleAssignmentProperties" = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Role assignment properties. Required."""

    @overload
    def __init__(
        self,
        *,
        properties: "_models.RoleAssignmentProperties",
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class RoleAssignmentProperties(_Model):
    """Role assignment properties.

    :ivar role_definition_id: The role definition ID used in the role assignment. Required.
    :vartype role_definition_id: str
    :ivar principal_id: The principal ID assigned to the role. This maps to the ID inside the
     Active Directory. It can point to a user, service principal, or security group. Required.
    :vartype principal_id: str
    """

    role_definition_id: str = rest_field(
        name="roleDefinitionId", visibility=["read", "create", "update", "delete", "query"]
    )
    """The role definition ID used in the role assignment. Required."""
    principal_id: str = rest_field(name="principalId", visibility=["read", "create", "update", "delete", "query"])
    """The principal ID assigned to the role. This maps to the ID inside the Active Directory. It can
     point to a user, service principal, or security group. Required."""

    @overload
    def __init__(
        self,
        *,
        role_definition_id: str,
        principal_id: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class RoleAssignmentPropertiesWithScope(_Model):
    """Role assignment properties with scope.

    :ivar scope: The role scope. Known values are: "/" and "/keys".
    :vartype scope: str or ~azure.keyvault.administration._generated.models.RoleScope
    :ivar role_definition_id: The role definition ID.
    :vartype role_definition_id: str
    :ivar principal_id: The principal ID.
    :vartype principal_id: str
    """

    scope: Optional[Union[str, "_models.RoleScope"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """The role scope. Known values are: \"/\" and \"/keys\"."""
    role_definition_id: Optional[str] = rest_field(
        name="roleDefinitionId", visibility=["read", "create", "update", "delete", "query"]
    )
    """The role definition ID."""
    principal_id: Optional[str] = rest_field(
        name="principalId", visibility=["read", "create", "update", "delete", "query"]
    )
    """The principal ID."""

    @overload
    def __init__(
        self,
        *,
        scope: Optional[Union[str, "_models.RoleScope"]] = None,
        role_definition_id: Optional[str] = None,
        principal_id: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class RoleDefinition(_Model):
    """Role definition.

    :ivar id: The role definition ID.
    :vartype id: str
    :ivar name: The role definition name.
    :vartype name: str
    :ivar type: The role definition type. "Microsoft.Authorization/roleDefinitions"
    :vartype type: str or ~azure.keyvault.administration._generated.models.RoleDefinitionType
    :ivar properties: Role definition properties.
    :vartype properties: ~azure.keyvault.administration._generated.models.RoleDefinitionProperties
    """

    id: Optional[str] = rest_field(visibility=["read"])
    """The role definition ID."""
    name: Optional[str] = rest_field(visibility=["read"])
    """The role definition name."""
    type: Optional[Union[str, "_models.RoleDefinitionType"]] = rest_field(visibility=["read"])
    """The role definition type. \"Microsoft.Authorization/roleDefinitions\""""
    properties: Optional["_models.RoleDefinitionProperties"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Role definition properties."""

    __flattened_items = ["role_name", "description", "role_type", "permissions", "assignable_scopes"]

    @overload
    def __init__(
        self,
        *,
        properties: Optional["_models.RoleDefinitionProperties"] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        _flattened_input = {k: kwargs.pop(k) for k in kwargs.keys() & self.__flattened_items}
        super().__init__(*args, **kwargs)
        for k, v in _flattened_input.items():
            setattr(self, k, v)

    def __getattr__(self, name: str) -> Any:
        if name in self.__flattened_items:
            if self.properties is None:
                return None
            return getattr(self.properties, name)
        raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{name}'")

    def __setattr__(self, key: str, value: Any) -> None:
        if key in self.__flattened_items:
            if self.properties is None:
                self.properties = self._attr_to_rest_field["properties"]._class_type()
            setattr(self.properties, key, value)
        else:
            super().__setattr__(key, value)


class RoleDefinitionCreateParameters(_Model):
    """Role definition create parameters.

    :ivar properties: Role definition properties. Required.
    :vartype properties: ~azure.keyvault.administration._generated.models.RoleDefinitionProperties
    """

    properties: "_models.RoleDefinitionProperties" = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Role definition properties. Required."""

    @overload
    def __init__(
        self,
        *,
        properties: "_models.RoleDefinitionProperties",
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class RoleDefinitionProperties(_Model):
    """Role definition properties.

    :ivar role_name: The role name.
    :vartype role_name: str
    :ivar description: The role definition description.
    :vartype description: str
    :ivar role_type: The role type. Known values are: "AKVBuiltInRole" and "CustomRole".
    :vartype role_type: str or ~azure.keyvault.administration._generated.models.RoleType
    :ivar permissions: Role definition permissions.
    :vartype permissions: list[~azure.keyvault.administration._generated.models.Permission]
    :ivar assignable_scopes: Role definition assignable scopes.
    :vartype assignable_scopes: list[str or
     ~azure.keyvault.administration._generated.models.RoleScope]
    """

    role_name: Optional[str] = rest_field(name="roleName", visibility=["read", "create", "update", "delete", "query"])
    """The role name."""
    description: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The role definition description."""
    role_type: Optional[Union[str, "_models.RoleType"]] = rest_field(
        name="type", visibility=["read", "create", "update", "delete", "query"]
    )
    """The role type. Known values are: \"AKVBuiltInRole\" and \"CustomRole\"."""
    permissions: Optional[list["_models.Permission"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Role definition permissions."""
    assignable_scopes: Optional[list[Union[str, "_models.RoleScope"]]] = rest_field(
        name="assignableScopes", visibility=["read", "create", "update", "delete", "query"]
    )
    """Role definition assignable scopes."""

    @overload
    def __init__(
        self,
        *,
        role_name: Optional[str] = None,
        description: Optional[str] = None,
        role_type: Optional[Union[str, "_models.RoleType"]] = None,
        permissions: Optional[list["_models.Permission"]] = None,
        assignable_scopes: Optional[list[Union[str, "_models.RoleScope"]]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class SASTokenParameter(_Model):
    """An authentication method and location for the operation.

    :ivar storage_resource_uri: Azure Blob storage container Uri. Required.
    :vartype storage_resource_uri: str
    :ivar token: The SAS token pointing to an Azure Blob storage container.
    :vartype token: str
    :ivar use_managed_identity: Indicates which authentication method should be used. If set to
     true, Managed HSM will use the configured user-assigned managed identity to authenticate with
     Azure Storage. Otherwise, a SAS token has to be specified.
    :vartype use_managed_identity: bool
    """

    storage_resource_uri: str = rest_field(
        name="storageResourceUri", visibility=["read", "create", "update", "delete", "query"]
    )
    """Azure Blob storage container Uri. Required."""
    token: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The SAS token pointing to an Azure Blob storage container."""
    use_managed_identity: Optional[bool] = rest_field(
        name="useManagedIdentity", visibility=["read", "create", "update", "delete", "query"]
    )
    """Indicates which authentication method should be used. If set to true, Managed HSM will use the
     configured user-assigned managed identity to authenticate with Azure Storage. Otherwise, a SAS
     token has to be specified."""

    @overload
    def __init__(
        self,
        *,
        storage_resource_uri: str,
        token: Optional[str] = None,
        use_managed_identity: Optional[bool] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class SelectiveKeyRestoreOperation(_Model):
    """Selective Key Restore operation.

    :ivar status: Status of the restore operation. Known values are: "InProgress", "Succeeded",
     "Canceled", and "Failed".
    :vartype status: str or ~azure.keyvault.administration._generated.models.OperationStatus
    :ivar status_details: The status details of restore operation.
    :vartype status_details: str
    :ivar error: Error encountered, if any, during the selective key restore operation.
    :vartype error: ~azure.keyvault.administration._generated.models.FullBackupOperationError
    :ivar job_id: Identifier for the selective key restore operation.
    :vartype job_id: str
    :ivar start_time: The start time of the restore operation.
    :vartype start_time: ~datetime.datetime
    :ivar end_time: The end time of the restore operation.
    :vartype end_time: ~datetime.datetime
    """

    status: Optional[Union[str, "_models.OperationStatus"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Status of the restore operation. Known values are: \"InProgress\", \"Succeeded\", \"Canceled\",
     and \"Failed\"."""
    status_details: Optional[str] = rest_field(
        name="statusDetails", visibility=["read", "create", "update", "delete", "query"]
    )
    """The status details of restore operation."""
    error: Optional["_models.FullBackupOperationError"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Error encountered, if any, during the selective key restore operation."""
    job_id: Optional[str] = rest_field(name="jobId", visibility=["read", "create", "update", "delete", "query"])
    """Identifier for the selective key restore operation."""
    start_time: Optional[datetime.datetime] = rest_field(
        name="startTime", visibility=["read", "create", "update", "delete", "query"], format="unix-timestamp"
    )
    """The start time of the restore operation."""
    end_time: Optional[datetime.datetime] = rest_field(
        name="endTime", visibility=["read", "create", "update", "delete", "query"], format="unix-timestamp"
    )
    """The end time of the restore operation."""

    @overload
    def __init__(
        self,
        *,
        status: Optional[Union[str, "_models.OperationStatus"]] = None,
        status_details: Optional[str] = None,
        error: Optional["_models.FullBackupOperationError"] = None,
        job_id: Optional[str] = None,
        start_time: Optional[datetime.datetime] = None,
        end_time: Optional[datetime.datetime] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class SelectiveKeyRestoreOperationParameters(_Model):
    """The authentication method and location for the selective key restore operation.

    :ivar sas_token_parameters: A user-provided SAS token to an Azure blob storage container.
     Required.
    :vartype sas_token_parameters:
     ~azure.keyvault.administration._generated.models.SASTokenParameter
    :ivar folder: The Folder name of the blob where the previous successful full backup was stored.
     Required.
    :vartype folder: str
    """

    sas_token_parameters: "_models.SASTokenParameter" = rest_field(
        name="sasTokenParameters", visibility=["read", "create", "update", "delete", "query"]
    )
    """A user-provided SAS token to an Azure blob storage container. Required."""
    folder: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The Folder name of the blob where the previous successful full backup was stored. Required."""

    @overload
    def __init__(
        self,
        *,
        sas_token_parameters: "_models.SASTokenParameter",
        folder: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class Setting(_Model):
    """A Key Vault account setting.

    :ivar name: The account setting to be updated. Required.
    :vartype name: str
    :ivar value: The value of the pool setting. Required.
    :vartype value: str
    :ivar type: The type specifier of the value. "boolean"
    :vartype type: str or ~azure.keyvault.administration._generated.models.SettingTypeEnum
    """

    name: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The account setting to be updated. Required."""
    value: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The value of the pool setting. Required."""
    type: Optional[Union[str, "_models.SettingTypeEnum"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """The type specifier of the value. \"boolean\""""

    @overload
    def __init__(
        self,
        *,
        name: str,
        value: str,
        type: Optional[Union[str, "_models.SettingTypeEnum"]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class SettingsListResult(_Model):
    """The settings list result.

    :ivar settings: A response message containing a list of account settings with their associated
     value.
    :vartype settings: list[~azure.keyvault.administration._generated.models.Setting]
    """

    settings: Optional[list["_models.Setting"]] = rest_field(visibility=["read"])
    """A response message containing a list of account settings with their associated value."""


class UpdateSettingRequest(_Model):
    """The update settings request object.

    :ivar value: The value of the pool setting. Required.
    :vartype value: str
    """

    value: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The value of the pool setting. Required."""

    @overload
    def __init__(
        self,
        *,
        value: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
import os

import pytest
from azure.keyvault.administration._internal.client_base import DEFAULT_VERSION
from devtools_testutils import AzureRecordedTestCase


class BaseClientPreparer(AzureRecordedTestCase):
    def __init__(self, **kwargs) -> None:
        hsm_playback_url = "https://managedhsmvaultname.managedhsm.azure.net"
        container_playback_uri = "https://storagename.blob.core.windows.net/container"
        playback_sas_token = "fake-sas"
        playback_ekm_host = "fake-ekm-host"
        playback_ekm_certificate = "ZmFrZS1jZXJ0LWRhdGE="

        if self.is_live:
            hsm = os.environ.get("AZURE_MANAGEDHSM_URL")
            self.managed_hsm_url = hsm if hsm else None
            storage_url = os.environ.get("BLOB_STORAGE_URL")
            container_name = os.environ.get("BLOB_CONTAINER_NAME")
            self.container_uri = f"{storage_url.rstrip('/')}/{container_name}"

            self.sas_token = os.environ.get("BLOB_STORAGE_SAS_TOKEN")
            self.ekm_host = os.environ.get("EKM_PROXY_HOST")
            self.ekm_certificate = os.environ.get("EKM_SERVER_CA_CERTIFICATE")

        else:
            self.managed_hsm_url = hsm_playback_url
            self.container_uri = container_playback_uri
            self.sas_token = playback_sas_token
            self.ekm_host = playback_ekm_host
            self.ekm_certificate = playback_ekm_certificate

        use_pwsh = os.environ.get("AZURE_TEST_USE_PWSH_AUTH", "false")
        use_cli = os.environ.get("AZURE_TEST_USE_CLI_AUTH", "false")
        use_vscode = os.environ.get("AZURE_TEST_USE_VSCODE_AUTH", "false")
        use_azd = os.environ.get("AZURE_TEST_USE_AZD_AUTH", "false")
        # Only set service principal credentials if user-based auth is not requested
        if use_pwsh == use_cli == use_vscode == use_azd == "false":
            self._set_mgmt_settings_real_values()

    def _skip_if_not_configured(self, api_version, **kwargs):
        if self.is_live and api_version != DEFAULT_VERSION:
            pytest.skip("This test only uses the default API version for live tests")
        if self.is_live and self.managed_hsm_url is None:
            pytest.skip("No HSM endpoint for live testing")

    def _set_mgmt_settings_real_values(self):
        if self.is_live:
            os.environ["AZURE_TENANT_ID"] = os.getenv("KEYVAULT_TENANT_ID", "")  # empty in pipelines
            os.environ["AZURE_CLIENT_ID"] = os.getenv("KEYVAULT_CLIENT_ID", "")  # empty in pipelines
            os.environ["AZURE_CLIENT_SECRET"] = os.getenv("KEYVAULT_CLIENT_SECRET", "")  # empty for user-based auth


class KeyVaultBackupClientPreparer(BaseClientPreparer):
    def __call__(self, fn):
        async def _preparer(test_class, api_version, **kwargs):
            self._skip_if_not_configured(api_version)
            kwargs["container_uri"] = self.container_uri
            kwargs["managed_hsm_url"] = self.managed_hsm_url
            client = self.create_backup_client(api_version=api_version, **kwargs)

            async with client:
                await fn(test_class, client, **kwargs)

        return _preparer

    def create_backup_client(self, **kwargs):
        from azure.keyvault.administration.aio import KeyVaultBackupClient

        credential = self.get_credential(KeyVaultBackupClient, is_async=True)
        return self.create_client_from_credential(
            KeyVaultBackupClient, credential=credential, vault_url=self.managed_hsm_url, **kwargs
        )


class KeyVaultBackupClientSasPreparer(BaseClientPreparer):
    def __call__(self, fn):
        async def _preparer(test_class, api_version, **kwargs):
            self._skip_if_not_configured(api_version)
            kwargs["container_uri"] = self.container_uri
            kwargs["sas_token"] = self.sas_token
            kwargs["managed_hsm_url"] = self.managed_hsm_url
            client = self.create_backup_client(api_version=api_version, **kwargs)

            async with client:
                await fn(test_class, client, **kwargs)

        return _preparer

    def create_backup_client(self, **kwargs):
        from azure.keyvault.administration.aio import KeyVaultBackupClient

        credential = self.get_credential(KeyVaultBackupClient, is_async=True)
        return self.create_client_from_credential(
            KeyVaultBackupClient, credential=credential, vault_url=self.managed_hsm_url, **kwargs
        )


class KeyVaultAccessControlClientPreparer(BaseClientPreparer):
    def __call__(self, fn):
        async def _preparer(test_class, api_version, **kwargs):
            self._skip_if_not_configured(api_version)
            client = self.create_access_control_client(api_version=api_version, **kwargs)

            async with client:
                await fn(test_class, client, **kwargs)

        return _preparer

    def create_access_control_client(self, **kwargs):
        from azure.keyvault.administration.aio import KeyVaultAccessControlClient

        credential = self.get_credential(KeyVaultAccessControlClient, is_async=True)
        return self.create_client_from_credential(
            KeyVaultAccessControlClient, credential=credential, vault_url=self.managed_hsm_url, **kwargs
        )


class KeyVaultSettingsClientPreparer(BaseClientPreparer):
    def __call__(self, fn):
        async def _preparer(test_class, api_version, **kwargs):
            self._skip_if_not_configured(api_version)
            client = self.create_access_control_client(api_version=api_version, **kwargs)

            async with client:
                await fn(test_class, client, **kwargs)

        return _preparer

    def create_access_control_client(self, **kwargs):
        from azure.keyvault.administration.aio import KeyVaultSettingsClient

        credential = self.get_credential(KeyVaultSettingsClient, is_async=True)
        return self.create_client_from_credential(
            KeyVaultSettingsClient, credential=credential, vault_url=self.managed_hsm_url, **kwargs
        )


class KeyVaultEkmClientPreparer(BaseClientPreparer):
    def __call__(self, fn):
        async def _preparer(test_class, api_version, **kwargs):
            self._skip_if_not_configured(api_version)
            kwargs["ekm_host"] = self.ekm_host
            kwargs["ekm_certificate"] = self.ekm_certificate
            client = self.create_ekm_client(api_version=api_version, **kwargs)

            async with client:
                await fn(test_class, client, **kwargs)

        return _preparer

    def create_ekm_client(self, **kwargs):
        from azure.keyvault.administration.aio import KeyVaultEkmClient

        credential = self.get_credential(KeyVaultEkmClient, is_async=True)
        return self.create_client_from_credential(
            KeyVaultEkmClient, credential=credential, vault_url=self.managed_hsm_url, **kwargs
        )



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
from typing import Any

from azure.core.async_paging import AsyncItemPaged, AsyncList
from azure.core.tracing.decorator import distributed_trace
from azure.core.tracing.decorator_async import distributed_trace_async

from .._generated.models import UpdateSettingRequest
from .._internal import AsyncKeyVaultClientBase
from .._models import KeyVaultSetting


class KeyVaultSettingsClient(AsyncKeyVaultClientBase):
    """Provides methods to update, get, and list Managed HSM account settings.

    :param str vault_url: URL of the vault on which the client will operate. This is also called the vault's "DNS Name".
        You should validate that this URL references a valid Key Vault or Managed HSM resource.
        See https://aka.ms/azsdk/blog/vault-uri for details.
    :param credential: An object which can provide an access token for the vault, such as a credential from
        :mod:`azure.identity.aio`
    :type credential: ~azure.core.credentials_async.AsyncTokenCredential

    :keyword api_version: Version of the service API to use. Defaults to the most recent.
    :paramtype api_version: ~azure.keyvault.administration.ApiVersion or str
    :keyword bool verify_challenge_resource: Whether to verify the authentication challenge resource matches the Key
        Vault or Managed HSM domain. Defaults to True.
    """

    # pylint:disable=protected-access

    @distributed_trace_async
    async def get_setting(self, name: str, **kwargs: Any) -> KeyVaultSetting:
        """Gets the setting with the specified name.

        :param str name: The name of the account setting.

        :returns: The account setting, as a :class:`~azure.keyvault.administration.KeyVaultSetting`.
        :rtype: ~azure.keyvault.administration.KeyVaultSetting
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        result = await self._client.get_setting(setting_name=name, **kwargs)
        return KeyVaultSetting._from_generated(result)

    @distributed_trace
    def list_settings(self, **kwargs: Any) -> AsyncItemPaged[KeyVaultSetting]:
        """Lists all account settings.

        :returns: A paged object containing the account's settings.
        :rtype: ~azure.core.async_paging.AsyncItemPaged[~azure.keyvault.administration.KeyVaultSetting]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        result = self._client.get_settings(*kwargs)

        # We don't actually get a paged response from the generated method, so we mock the typical iteration methods
        async def get_next(_=None):
            # There's only one page of results (the `get_settings` result), so we return the awaited result directly
            return await result

        async def extract_data(pipeline_response):
            # `pipeline_response` is the awaited `get_settings` result that we returned in `get_next`
            converted_result = [KeyVaultSetting._from_generated(setting) for setting in pipeline_response.settings]
            return None, AsyncList(converted_result)

        return AsyncItemPaged(get_next, extract_data)

    @distributed_trace_async
    async def update_setting(self, setting: KeyVaultSetting, **kwargs: Any) -> KeyVaultSetting:
        """Updates the named account setting with the provided value.

        :param setting: A azure.keyvault.administration.KeyVaultSetting to update. The account setting with
            the provided name will be updated to have the provided value.
        :type setting: ~azure.keyvault.administration.KeyVaultSetting

        :returns: The updated account setting, as a :class:`~azure.keyvault.administration.KeyVaultSetting`.
        :rtype: ~azure.keyvault.administration.KeyVaultSetting
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        parameters = UpdateSettingRequest(value=setting.value)
        result = await self._client.update_setting(setting_name=setting.name, parameters=parameters, **kwargs)
        return KeyVaultSetting._from_generated(result)

    async def __aenter__(self) -> "KeyVaultSettingsClient":
        await self._client.__aenter__()
        return self



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
import asyncio
import os

from azure.keyvault.administration.aio import KeyVaultBackupClient
from azure.identity.aio import ManagedIdentityCredential

# ----------------------------------------------------------------------------------------------------------
# Prerequisites:
# 1. A managed HSM (https://learn.microsoft.com/azure/key-vault/managed-hsm/quick-create-cli)
#
# 2. azure-keyvault-administration and azure-identity libraries (pip install these)
#
# 3. Set environment variable MANAGED_HSM_URL with the URL of your managed HSM
#
# 4. A user-assigned managed identity that has access to your managed HSM. For more information about how to create a
#    user-assigned managed identity, refer to
#    https://learn.microsoft.com/entra/identity/managed-identities-azure-resources/overview
#
# 5. A storage account, that your managed identity has access to, containing a blob storage container
#    (See https://learn.microsoft.com/azure/storage/blobs/storage-blobs-introduction)
#
# 6. Set environment variables CONTAINER_URL and CLIENT_ID, corresponding to your blob container's URI and your
#    user-assigned managed identity's client ID, respectively.
#
# For more details, see https://learn.microsoft.com/azure/key-vault/managed-hsm/backup-restore
#
# ----------------------------------------------------------------------------------------------------------
# Sample - demonstrates full backup and restore operations for Managed HSM
#
# 1. Perform a full backup (begin_backup)
#
# 2. Perform a full restore (begin_restore)
# ----------------------------------------------------------------------------------------------------------


async def run_sample():
    MANAGED_HSM_URL = os.environ["MANAGED_HSM_URL"]
    CONTAINER_URL = os.environ["CONTAINER_URL"]
    MANAGED_IDENTITY_CLIENT_ID = os.environ["CLIENT_ID"]

    # Instantiate a backup client that will be used to call the service.
    # Here we use the DefaultAzureCredential, but any azure-identity credential can be used.
    credential = ManagedIdentityCredential(client_id=MANAGED_IDENTITY_CLIENT_ID)
    client = KeyVaultBackupClient(vault_url=MANAGED_HSM_URL, credential=credential)

    # Let's back up the vault with begin_backup, which returns a poller. Calling result() on the poller will return
    # a KeyVaultBackupResult that contains the URL of the backup after the operation completes. Calling wait() on
    # the poller will wait until the operation is complete.
    print("\n.. Back up the vault")
    backup_poller = await client.begin_backup(CONTAINER_URL, use_managed_identity=True)
    backup_result = await backup_poller.result()
    print("Vault backed up successfully.")
    assert backup_result.folder_url

    # Now let's the vault by calling begin_restore, which also returns a poller. Calling result() on the poller will
    # return None after the operation completes. Calling wait() on the poller will wait until the operation is
    # complete. To restore a single key from the backed up vault instead, pass the key_name keyword argument.
    print("\n.. Restore the full vault")
    restore_poller = await client.begin_restore(backup_result.folder_url, use_managed_identity=True)
    await restore_poller.wait()
    print("Vault restored successfully.")

    await credential.close()
    await client.close()


if __name__ == "__main__":
    asyncio.run(run_sample())



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
import base64
from typing import TYPE_CHECKING

from azure.core.polling.base_polling import LROBasePolling, OperationFailed, OperationResourcePolling

if TYPE_CHECKING:
    from azure.core.pipeline import PipelineResponse
    from azure.core.rest import HttpResponse


class KeyVaultBackupClientPolling(OperationResourcePolling):
    def __init__(self) -> None:
        self._polling_url = ""
        super().__init__(operation_location_header="azure-asyncoperation")

    def get_polling_url(self) -> str:
        return self._polling_url

    def get_final_get_url(self, pipeline_response: "PipelineResponse") -> None:
        return None

    def set_initial_status(self, pipeline_response: "PipelineResponse") -> str:
        response = pipeline_response.http_response  # type: HttpResponse
        self._polling_url = response.headers["azure-asyncoperation"]

        if response.status_code in {200, 201, 202, 204}:
            return self.get_status(pipeline_response)
        raise OperationFailed("Operation failed or canceled")


class KeyVaultBackupClientPollingMethod(LROBasePolling):
    def get_continuation_token(self) -> str:
        """
        Get a continuation token to resume the polling later.

        :return: A continuation token.
        :rtype: str
        """
        # Because of the operation structure, we need to use a "continuation token" that is just the status URL.
        # This URL can then be used to fetch the status of the operation when resuming, at which point a genuine
        # continuation token will be created from the response and provided to Core.
        return base64.b64encode(self._operation.get_polling_url().encode()).decode("ascii")



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
import base64
import functools
import json
from typing import Any, Callable, Optional, overload
from urllib.parse import urlparse

from typing_extensions import Literal

from azure.core.pipeline import PipelineResponse
from azure.core.polling import LROPoller
from azure.core.tracing.decorator import distributed_trace

from ._generated.models import PreBackupOperationParameters, PreRestoreOperationParameters, SASTokenParameter
from ._models import KeyVaultBackupResult
from ._internal import KeyVaultClientBase, parse_folder_url
from ._internal.polling import KeyVaultBackupClientPolling, KeyVaultBackupClientPollingMethod


def _parse_status_url(url: str) -> str:
    parsed = urlparse(url)
    job_id = parsed.path.split("/")[2]
    return job_id


def _get_continuation_token(pipeline_response: PipelineResponse) -> str:
    """Returns an opaque token which can be used by the user to rehydrate/restart the LRO.

    Saves the state of the LRO based on a status response so that polling can be resumed from that context. Because
    the service has different operations for backup/restore starting vs. status checking, the caller is expected to
    first use the status URL from the initial response to make a status request and then pass that status response to
    this function to be serialized into a continuation token.

    :param pipeline_response: The pipeline response of the operation status request.
    :type pipeline_response: ~azure.core.pipeline.PipelineResponse
    :returns: An opaque continuation token that can be provided to Core to rehydrate the LRO.
    :rtype: str
    """
    # Headers needed for LRO rehydration - use an allowlist approach for security
    lro_headers = {"azure-asyncoperation", "operation-location", "location", "content-type", "retry-after"}
    response = pipeline_response.http_response
    filtered_headers = {k: v for k, v in response.headers.items() if k.lower() in lro_headers}

    request = response.request
    # Serialize the essential parts of the PipelineResponse to JSON.
    if request:
        request_headers = {}
        # Preserve x-ms-client-request-id for request correlation
        if "x-ms-client-request-id" in request.headers:
            request_headers["x-ms-client-request-id"] = request.headers["x-ms-client-request-id"]
        request_state = {
            "method": request.method,
            "url": request.url,
            "headers": request_headers,
        }
    else:
        request_state = None

    # Use a versioned token schema: {"version": <int>, "data": <your state>}
    # This allows for future compatibility checking when deserializing
    token = {
        "version": 1,
        "data": {
            "request": request_state,
            "response": {
                "status_code": response.status_code,
                "headers": filtered_headers,
                "content": base64.b64encode(response.content).decode("ascii"),
            },
        },
    }
    return base64.b64encode(json.dumps(token, separators=(",", ":")).encode("utf-8")).decode("ascii")


class KeyVaultBackupClient(KeyVaultClientBase):
    """Performs Key Vault backup and restore operations.

    :param str vault_url: URL of the vault on which the client will operate. This is also called the vault's "DNS Name".
        You should validate that this URL references a valid Key Vault or Managed HSM resource.
        See https://aka.ms/azsdk/blog/vault-uri for details.
    :param credential: An object which can provide an access token for the vault, such as a credential from
        :mod:`azure.identity`
    :type credential: ~azure.core.credentials.TokenCredential

    :keyword api_version: Version of the service API to use. Defaults to the most recent.
    :paramtype api_version: ~azure.keyvault.administration.ApiVersion or str
    :keyword bool verify_challenge_resource: Whether to verify the authentication challenge resource matches the Key
        Vault or Managed HSM domain. Defaults to True.
    """

    def _use_continuation_token(self, continuation_token: str, status_method: Callable) -> str:
        status_url = base64.b64decode(continuation_token.encode()).decode("ascii")
        try:
            job_id = _parse_status_url(status_url)
        except Exception as ex:  # pylint: disable=broad-except
            raise ValueError(
                "The provided continuation_token is malformed. A valid token can be obtained from the "
                + "operation poller's continuation_token() method"
            ) from ex

        pipeline_response = status_method(job_id=job_id, cls=lambda pipeline_response, _, __: pipeline_response)
        if "azure-asyncoperation" not in pipeline_response.http_response.headers:
            pipeline_response.http_response.headers["azure-asyncoperation"] = status_url
        return _get_continuation_token(pipeline_response)

    @overload
    def begin_backup(
        self,
        blob_storage_url: str,
        *,
        use_managed_identity: Literal[True],
        continuation_token: Optional[str] = None,
        **kwargs: Any,
    ) -> LROPoller[KeyVaultBackupResult]: ...

    @overload
    def begin_backup(
        self,
        blob_storage_url: str,
        *,
        sas_token: str,
        continuation_token: Optional[str] = None,
        **kwargs: Any,
    ) -> LROPoller[KeyVaultBackupResult]: ...

    # Disabling pylint checks because they don't correctly handle overloads
    @distributed_trace
    def begin_backup(  # pylint: disable=docstring-missing-param,docstring-keyword-should-match-keyword-only
        self, blob_storage_url: str, *args: str, **kwargs: Any
    ) -> LROPoller[KeyVaultBackupResult]:
        """Begin a full backup of the Key Vault.

        :param str blob_storage_url: URL of the blob storage container in which the backup will be stored, for example
            https://<account>.blob.core.windows.net/backup.

        :keyword str sas_token: Optional Shared Access Signature (SAS) token to authorize access to the blob. Required
            unless `use_managed_identity` is set to True.
        :keyword use_managed_identity: Indicates which authentication method should be used. If set to True, Managed HSM
            will use the configured user-assigned managed identity to authenticate with Azure Storage. Otherwise, a SAS
            token has to be specified.
        :paramtype use_managed_identity: bool
        :keyword str continuation_token: A continuation token to restart polling from a saved state.

        :returns: An :class:`~azure.core.polling.LROPoller` instance. Call `result()` on this object to wait for the
            operation to complete and get a :class:`KeyVaultBackupResult`.
        :rtype: ~azure.core.polling.LROPoller[~azure.keyvault.administration.KeyVaultBackupResult]

        Example:
            .. literalinclude:: ../tests/test_examples_administration.py
                :start-after: [START begin_backup]
                :end-before: [END begin_backup]
                :language: python
                :caption: Create a vault backup
                :dedent: 8
        """
        polling_interval = kwargs.pop("_polling_interval", 5)
        continuation_token = kwargs.pop("continuation_token", None)
        use_managed_identity = kwargs.pop("use_managed_identity", False)
        # `sas_token` was formerly a required positional parameter
        try:
            sas_token: Optional[str] = args[0]
        except IndexError:
            sas_token = kwargs.pop("sas_token", None)
        sas_parameter = self._models.SASTokenParameter(
            storage_resource_uri=blob_storage_url, token=sas_token, use_managed_identity=use_managed_identity
        )

        status_response = None
        if continuation_token:
            status_response = self._use_continuation_token(continuation_token, self._client.full_backup_status)

        return self._client.begin_full_backup(
            azure_storage_blob_container_uri=sas_parameter,
            cls=KeyVaultBackupResult._from_generated,  # pylint: disable=protected-access
            continuation_token=status_response,
            polling=KeyVaultBackupClientPollingMethod(
                lro_algorithms=[KeyVaultBackupClientPolling()], timeout=polling_interval, **kwargs
            ),
            **kwargs,
        )

    @overload
    def begin_restore(
        self,
        folder_url: str,
        *,
        use_managed_identity: Literal[True],
        key_name: Optional[str] = None,
        continuation_token: Optional[str] = None,
        **kwargs: Any,
    ) -> LROPoller[None]: ...

    @overload
    def begin_restore(
        self,
        folder_url: str,
        *,
        sas_token: str,
        key_name: Optional[str] = None,
        continuation_token: Optional[str] = None,
        **kwargs: Any,
    ) -> LROPoller[None]: ...

    # Disabling pylint checks because they don't correctly handle overloads
    @distributed_trace
    def begin_restore(  # pylint: disable=docstring-missing-param,docstring-keyword-should-match-keyword-only
        self, folder_url: str, *args: str, **kwargs: Any
    ) -> LROPoller[None]:
        """Restore a Key Vault backup.

        This method restores either a complete Key Vault backup or when ``key_name`` has a value, a single key.

        :param str folder_url: URL of the blob holding the backup. This would be the `folder_url` of a
            :class:`KeyVaultBackupResult` returned by :func:`begin_backup`, for example
            https://<account>.blob.core.windows.net/backup/mhsm-account-2020090117323313

        :keyword str sas_token: Optional Shared Access Signature (SAS) token to authorize access to the blob. Required
            unless `use_managed_identity` is set to True.
        :keyword use_managed_identity: Indicates which authentication method should be used. If set to True, Managed HSM
            will use the configured user-assigned managed identity to authenticate with Azure Storage. Otherwise, a SAS
            token has to be specified.
        :paramtype use_managed_identity: bool
        :keyword str key_name: Name of a single key in the backup. When set, only this key will be restored.
        :keyword str continuation_token: A continuation token to restart polling from a saved state.

        :returns: An :class:`~azure.core.polling.LROPoller` instance. Call `wait()` or `result()` on this object to wait
            for the operation to complete (the return value is None in either case).
        :rtype: ~azure.core.polling.LROPoller

        Examples:
            .. literalinclude:: ../tests/test_examples_administration.py
                :start-after: [START begin_restore]
                :end-before: [END begin_restore]
                :language: python
                :caption: Restore a vault backup
                :dedent: 8

            .. literalinclude:: ../tests/test_examples_administration.py
                :start-after: [START begin_selective_restore]
                :end-before: [END begin_selective_restore]
                :language: python
                :caption: Restore a single key
                :dedent: 8
        """
        polling_interval = kwargs.pop("_polling_interval", 5)
        continuation_token = kwargs.pop("continuation_token", None)
        key_name = kwargs.pop("key_name", None)
        use_managed_identity = kwargs.pop("use_managed_identity", False)
        # `sas_token` was formerly a required positional parameter
        try:
            sas_token: Optional[str] = args[0]
        except IndexError:
            sas_token = kwargs.pop("sas_token", None)

        status_response = None
        if continuation_token:
            status_response = self._use_continuation_token(continuation_token, self._client.restore_status)

        container_url, folder_name = parse_folder_url(folder_url)
        sas_parameter = self._models.SASTokenParameter(
            storage_resource_uri=container_url, token=sas_token, use_managed_identity=use_managed_identity
        )
        polling = KeyVaultBackupClientPollingMethod(
            lro_algorithms=[KeyVaultBackupClientPolling()], timeout=polling_interval, **kwargs
        )

        if key_name:
            client_method = functools.partial(self._client.begin_selective_key_restore_operation, key_name=key_name)
            restore_details = self._models.SelectiveKeyRestoreOperationParameters(
                sas_token_parameters=sas_parameter, folder=folder_name
            )
        else:
            client_method = self._client.begin_full_restore_operation
            restore_details = self._models.RestoreOperationParameters(
                sas_token_parameters=sas_parameter, folder_to_restore=folder_name
            )

        return client_method(
            restore_blob_details=restore_details,
            cls=lambda *_: None,  # poller.result() returns None
            continuation_token=status_response,
            polling=polling,
            **kwargs,
        )

    @overload
    def begin_pre_backup(
        self,
        blob_storage_url: str,
        *,
        use_managed_identity: Literal[True],
        continuation_token: Optional[str] = None,
        **kwargs: Any,
    ) -> LROPoller[None]: ...

    @overload
    def begin_pre_backup(
        self,
        blob_storage_url: str,
        *,
        sas_token: str,
        continuation_token: Optional[str] = None,
        **kwargs: Any,
    ) -> LROPoller[None]: ...

    @distributed_trace
    def begin_pre_backup(  # pylint: disable=docstring-keyword-should-match-keyword-only
        self, blob_storage_url: str, **kwargs: Any
    ) -> LROPoller[None]:
        """Initiates a pre-backup check of whether a full Key Vault backup can be performed.

        If the pre-backup check fails, calling `wait()` on the returned poller will raise an error. Otherwise, a full
        backup can be performed.

        :param str blob_storage_url: URL of the blob storage container in which the backup will be stored, for example
            https://<account>.blob.core.windows.net/backup.

        :keyword str sas_token: Optional Shared Access Signature (SAS) token to authorize access to the blob. Required
            unless `use_managed_identity` is set to True.
        :keyword use_managed_identity: Indicates which authentication method should be used. If set to True, Managed HSM
            will use the configured user-assigned managed identity to authenticate with Azure Storage. Otherwise, a SAS
            token has to be specified.
        :paramtype use_managed_identity: bool
        :keyword str continuation_token: A continuation token to restart polling from a saved state.

        :returns: An :class:`~azure.core.polling.LROPoller` instance. Call `wait()` on this object to wait for the
            operation to complete. If the check fails, an error will be raised.
        :rtype: ~azure.core.polling.LROPoller[None]
        """
        polling_interval: int = kwargs.pop("_polling_interval", 5)
        continuation_token: Optional[str] = kwargs.pop("continuation_token", None)
        use_managed_identity: bool = kwargs.pop("use_managed_identity", False)
        sas_token: Optional[str] = kwargs.pop("sas_token", None)

        parameters: PreBackupOperationParameters = PreBackupOperationParameters(
            storage_resource_uri=blob_storage_url, token=sas_token, use_managed_identity=use_managed_identity
        )
        status_response: Optional[str] = None
        if continuation_token:
            status_response = self._use_continuation_token(continuation_token, self._client.full_backup_status)

        return self._client.begin_pre_full_backup(
            pre_backup_operation_parameters=parameters,
            cls=lambda *_: None,  # poller.result() returns None
            polling=KeyVaultBackupClientPollingMethod(
                lro_algorithms=[KeyVaultBackupClientPolling()], timeout=polling_interval, **kwargs
            ),
            continuation_token=status_response,
            **kwargs,
        )

    @overload
    def begin_pre_restore(
        self,
        folder_url: str,
        *,
        use_managed_identity: Literal[True],
        continuation_token: Optional[str] = None,
        **kwargs: Any,
    ) -> LROPoller[None]: ...

    @overload
    def begin_pre_restore(
        self,
        folder_url: str,
        *,
        sas_token: str,
        continuation_token: Optional[str] = None,
        **kwargs: Any,
    ) -> LROPoller[None]: ...

    @distributed_trace
    def begin_pre_restore(  # pylint: disable=docstring-keyword-should-match-keyword-only
        self, folder_url: str, **kwargs: Any
    ) -> LROPoller[None]:
        """Initiates a pre-restore check of whether a full Key Vault restore can be performed.

        If the pre-restore check fails, calling `wait()` on the returned poller will raise an error. Otherwise, a full
        restore can be performed.

        :param str folder_url: URL of the blob holding the backup. This would be the `folder_url` of a
            :class:`KeyVaultBackupResult` returned by :func:`begin_backup`, for example
            https://<account>.blob.core.windows.net/backup/mhsm-account-2020090117323313

        :keyword str sas_token: Optional Shared Access Signature (SAS) token to authorize access to the blob. Required
            unless `use_managed_identity` is set to True.
        :keyword use_managed_identity: Indicates which authentication method should be used. If set to True, Managed HSM
            will use the configured user-assigned managed identity to authenticate with Azure Storage. Otherwise, a SAS
            token has to be specified.
        :paramtype use_managed_identity: bool
        :keyword str continuation_token: A continuation token to restart polling from a saved state.

        :returns: An :class:`~azure.core.polling.LROPoller` instance. Call `wait()` on this object to wait for the
            operation to complete. If the check fails, an error will be raised.
        :rtype: ~azure.core.polling.LROPoller[None]
        """
        polling_interval: int = kwargs.pop("_polling_interval", 5)
        continuation_token: Optional[str] = kwargs.pop("continuation_token", None)
        use_managed_identity: bool = kwargs.pop("use_managed_identity", False)
        sas_token: Optional[str] = kwargs.pop("sas_token", None)

        container_url, folder_name = parse_folder_url(folder_url)
        sas_parameter: SASTokenParameter = SASTokenParameter(
            storage_resource_uri=container_url, token=sas_token, use_managed_identity=use_managed_identity
        )
        parameters: PreRestoreOperationParameters = PreRestoreOperationParameters(
            folder_to_restore=folder_name, sas_token_parameters=sas_parameter
        )
        status_response: Optional[str] = None
        if continuation_token:
            status_response = self._use_continuation_token(continuation_token, self._client.restore_status)

        return self._client.begin_pre_full_restore_operation(
            pre_restore_operation_parameters=parameters,
            cls=lambda *_: None,  # poller.result() returns None
            polling=KeyVaultBackupClientPollingMethod(
                lro_algorithms=[KeyVaultBackupClientPolling()], timeout=polling_interval, **kwargs
            ),
            continuation_token=status_response,
            **kwargs,
        )

    def __enter__(self) -> "KeyVaultBackupClient":
        self._client.__enter__()
        return self



# pylint: disable=line-too-long,useless-suppression
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
import asyncio
import os

from azure.keyvault.administration import KeyVaultSetting, KeyVaultSettingType
from azure.keyvault.administration.aio import KeyVaultSettingsClient
from azure.identity.aio import DefaultAzureCredential

# ----------------------------------------------------------------------------------------------------------
# Prerequisites:
# 1. A managed HSM (https://learn.microsoft.com/azure/key-vault/managed-hsm/quick-create-cli)
#
# 2. azure-keyvault-administration and azure-identity libraries (pip install these)
#
# 3. Set environment variable MANAGED_HSM_URL with the URL of your managed HSM
#
# 4. Set up your environment to use azure-identity's DefaultAzureCredential. For more information about how to configure
#    the DefaultAzureCredential, refer to https://aka.ms/azsdk/python/identity/docs#azure.identity.DefaultAzureCredential
#
# ----------------------------------------------------------------------------------------------------------
# Sample - demonstrates Key Vault setting management operations for Managed HSM
#
# 1. List all settings (list_settings)
#
# 2. Update a setting (update_setting)
# ----------------------------------------------------------------------------------------------------------


async def run_sample():
    MANAGED_HSM_URL = os.environ["MANAGED_HSM_URL"]

    # Instantiate an access control client that will be used to call the service.
    # Here we use the DefaultAzureCredential, but any azure-identity credential can be used.
    credential = DefaultAzureCredential()
    client = KeyVaultSettingsClient(vault_url=MANAGED_HSM_URL, credential=credential)

    # First, let's fetch the settings that apply to our Managed HSM
    # Each setting has a name, value, and type (for example, KeyVaultSettingType.BOOLEAN)
    print("\n.. List Key Vault settings")
    settings = client.list_settings()
    boolean_setting = None
    async for setting in settings:
        if setting.setting_type == KeyVaultSettingType.BOOLEAN:
            boolean_setting = setting
        print(f"{setting.name}: {setting.value} (type: {setting.setting_type})")
    assert boolean_setting

    # Now, let's flip the value of a boolean setting
    # The `value` property is a string, but you can get the value of a boolean setting as a bool with `getboolean`
    print(f"\n.. Update value of {boolean_setting.name}")
    opposite_value = KeyVaultSetting(name=boolean_setting.name, value=not setting.getboolean())
    updated_setting = await client.update_setting(opposite_value)
    print(f"{boolean_setting.name} updated successfully.")
    print(f"Old value: {boolean_setting.value}. New value: {updated_setting.value}")

    # Finally, let's restore the setting's old value
    print(f"\n.. Restore original value of {boolean_setting.name}")
    await client.update_setting(boolean_setting)
    print(f"{boolean_setting.name} updated successfully.")

    await client.close()
    await credential.close()


if __name__ == "__main__":
    asyncio.run(run_sample())



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
from ._access_control_client import KeyVaultAccessControlClient
from ._backup_client import KeyVaultBackupClient
from ._ekm_client import KeyVaultEkmClient
from ._settings_client import KeyVaultSettingsClient

__all__ = [
    "KeyVaultAccessControlClient",
    "KeyVaultBackupClient",
    "KeyVaultEkmClient",
    "KeyVaultSettingsClient",
]



# pylint: disable=line-too-long,useless-suppression
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
import asyncio
import os

from azure.keyvault.administration import KeyVaultDataAction, KeyVaultPermission, KeyVaultRoleScope
from azure.keyvault.administration.aio import KeyVaultAccessControlClient
from azure.identity.aio import DefaultAzureCredential

# ----------------------------------------------------------------------------------------------------------
# Prerequisites:
# 1. A managed HSM (https://learn.microsoft.com/azure/key-vault/managed-hsm/quick-create-cli)
#
# 2. azure-keyvault-administration and azure-identity libraries (pip install these)
#
# 3. Set environment variable MANAGED_HSM_URL with the URL of your managed HSM
#
# 4. Set up your environment to use azure-identity's DefaultAzureCredential. For more information about how to configure
#    the DefaultAzureCredential, refer to https://aka.ms/azsdk/python/identity/docs#azure.identity.DefaultAzureCredential
#
# ----------------------------------------------------------------------------------------------------------
# Sample - demonstrates role definition and assignment operations for Managed HSM
#
# 1. Create a role definition (set_role_definition)
#
# 2. Update a role definition (set_role_definition)
#
# 3. Create a role assignment (create_role_assignment)
#
# 4. Delete a role assignment (delete_role_assignment)
#
# 5. Delete a role definition (delete_role_definition)
# ----------------------------------------------------------------------------------------------------------


async def run_sample():
    MANAGED_HSM_URL = os.environ["MANAGED_HSM_URL"]

    # Instantiate an access control client that will be used to call the service.
    # Here we use the DefaultAzureCredential, but any azure-identity credential can be used.
    credential = DefaultAzureCredential()
    client = KeyVaultAccessControlClient(vault_url=MANAGED_HSM_URL, credential=credential)

    # Let's first create a custom role definition. This role permits creating keys in a Managed HSM.
    # We'll provide a friendly role name, and let a unique role definition name (a GUID) be generated for us.
    print("\n.. Create a role definition")
    role_name = "customRole"
    scope = KeyVaultRoleScope.GLOBAL
    permissions = [KeyVaultPermission(data_actions=[KeyVaultDataAction.CREATE_HSM_KEY])]
    role_definition = await client.set_role_definition(scope=scope, role_name=role_name, permissions=permissions)
    print(f"Role definition '{role_definition.role_name}' created successfully.")

    # Let's update our role definition to allow reading keys, but not allow creating keys.
    # To update an existing definition, pass the name keyword argument to set_role_definition. This is the unique
    # name of the role definition, which is stored in KeyVaultRoleDefinition.name.
    print("\n.. Update a role definition")
    new_permissions = [
        KeyVaultPermission(
            data_actions=[KeyVaultDataAction.READ_HSM_KEY], not_data_actions=[KeyVaultDataAction.CREATE_HSM_KEY]
        )
    ]
    unique_definition_name = role_definition.name
    updated_definition = await client.set_role_definition(
        scope=scope, name=unique_definition_name, role_name=role_name, permissions=new_permissions
    )
    print(f"Role definition '{updated_definition.role_name}' updated successfully.")

    # Now let's create a role assignment to apply our role definition to our service principal.
    # Since we don't provide the name keyword argument to create_role_definition, a unique role assignment name
    # (a GUID) is generated for us.
    print("\n.. Create a role assignment")
    principal_id = os.environ["AZURE_CLIENT_ID"]
    definition_id = updated_definition.id
    assert definition_id
    role_assignment = await client.create_role_assignment(
        scope=scope, definition_id=definition_id, principal_id=principal_id
    )
    assert role_assignment.name
    print(f"Role assignment {role_assignment.name} created successfully.")

    # Let's delete the role assignment.
    print("\n.. Delete a role assignment")
    await client.delete_role_assignment(scope=scope, name=role_assignment.name)
    print("Role assignment deleted successfully.")

    # Finally, let's delete the role definition as well.
    print("\n.. Delete a role definition")
    await client.delete_role_definition(scope=scope, name=definition_id)
    print("Role definition deleted successfully.")

    await credential.close()
    await client.close()


if __name__ == "__main__":
    asyncio.run(run_sample())



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class DataAction(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Supported permissions for data actions."""

    READ_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/read/action"
    """Read HSM key metadata."""
    WRITE_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/write/action"
    """Update an HSM key."""
    READ_DELETED_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/deletedKeys/read/action"
    """Read deleted HSM key."""
    RECOVER_DELETED_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/deletedKeys/recover/action"
    """Recover deleted HSM key."""
    BACKUP_HSM_KEYS = "Microsoft.KeyVault/managedHsm/keys/backup/action"
    """Backup HSM keys."""
    RESTORE_HSM_KEYS = "Microsoft.KeyVault/managedHsm/keys/restore/action"
    """Restore HSM keys."""
    DELETE_ROLE_ASSIGNMENT = "Microsoft.KeyVault/managedHsm/roleAssignments/delete/action"
    """Delete role assignment."""
    GET_ROLE_ASSIGNMENT = "Microsoft.KeyVault/managedHsm/roleAssignments/read/action"
    """Get role assignment."""
    WRITE_ROLE_ASSIGNMENT = "Microsoft.KeyVault/managedHsm/roleAssignments/write/action"
    """Create or update role assignment."""
    READ_ROLE_DEFINITION = "Microsoft.KeyVault/managedHsm/roleDefinitions/read/action"
    """Get role definition."""
    WRITE_ROLE_DEFINITION = "Microsoft.KeyVault/managedHsm/roleDefinitions/write/action"
    """Create or update role definition."""
    DELETE_ROLE_DEFINITION = "Microsoft.KeyVault/managedHsm/roleDefinitions/delete/action"
    """Delete role definition."""
    ENCRYPT_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/encrypt/action"
    """Encrypt using an HSM key."""
    DECRYPT_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/decrypt/action"
    """Decrypt using an HSM key."""
    WRAP_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/wrap/action"
    """Wrap using an HSM key."""
    UNWRAP_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/unwrap/action"
    """Unwrap using an HSM key."""
    SIGN_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/sign/action"
    """Sign using an HSM key."""
    VERIFY_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/verify/action"
    """Verify using an HSM key."""
    CREATE_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/create"
    """Create an HSM key."""
    DELETE_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/delete"
    """Delete an HSM key."""
    EXPORT_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/export/action"
    """Export an HSM key."""
    RELEASE_KEY = "Microsoft.KeyVault/managedHsm/keys/release/action"
    """Release an HSM key using Secure Key Release."""
    IMPORT_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/import/action"
    """Import an HSM key."""
    PURGE_DELETED_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/deletedKeys/delete"
    """Purge a deleted HSM key."""
    DOWNLOAD_HSM_SECURITY_DOMAIN = "Microsoft.KeyVault/managedHsm/securitydomain/download/action"
    """Download an HSM security domain."""
    DOWNLOAD_HSM_SECURITY_DOMAIN_STATUS = "Microsoft.KeyVault/managedHsm/securitydomain/download/read"
    """Check status of HSM security domain download."""
    UPLOAD_HSM_SECURITY_DOMAIN = "Microsoft.KeyVault/managedHsm/securitydomain/upload/action"
    """Upload an HSM security domain."""
    READ_HSM_SECURITY_DOMAIN_STATUS = "Microsoft.KeyVault/managedHsm/securitydomain/upload/read"
    """Check the status of the HSM security domain exchange file."""
    READ_HSM_SECURITY_DOMAIN_TRANSFER_KEY = "Microsoft.KeyVault/managedHsm/securitydomain/transferkey/read"
    """Download an HSM security domain transfer key."""
    START_HSM_BACKUP = "Microsoft.KeyVault/managedHsm/backup/start/action"
    """Start an HSM backup."""
    START_HSM_RESTORE = "Microsoft.KeyVault/managedHsm/restore/start/action"
    """Start an HSM restore."""
    READ_HSM_BACKUP_STATUS = "Microsoft.KeyVault/managedHsm/backup/status/action"
    """Read an HSM backup status."""
    READ_HSM_RESTORE_STATUS = "Microsoft.KeyVault/managedHsm/restore/status/action"
    """Read an HSM restore status."""
    RANDOM_NUMBERS_GENERATE = "Microsoft.KeyVault/managedHsm/rng/action"
    """Generate random numbers."""


class OperationStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The status of a long-running operation."""

    IN_PROGRESS = "InProgress"
    """The operation is in progress."""
    SUCCEEDED = "Succeeded"
    """The operation successfully completed."""
    CANCELED = "Canceled"
    """The operation was canceled."""
    FAILED = "Failed"
    """The operation failed."""


class RoleDefinitionType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The role definition type."""

    MICROSOFT_AUTHORIZATION_ROLE_DEFINITIONS = "Microsoft.Authorization/roleDefinitions"
    """Microsoft-defined role definitions."""


class RoleScope(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The role scope."""

    GLOBAL = "/"
    """Global scope."""
    KEYS = "/keys"
    """Keys scope."""


class RoleType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The role type."""

    BUILT_IN_ROLE = "AKVBuiltInRole"
    """Built in role."""
    CUSTOM_ROLE = "CustomRole"
    """Custom role."""


class SettingTypeEnum(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type specifier of the value."""

    BOOLEAN = "boolean"
    """A boolean setting value."""



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
import asyncio

import pytest
from azure.core.exceptions import ResourceExistsError
from azure.keyvault.administration._internal.client_base import DEFAULT_VERSION
from devtools_testutils import set_bodiless_matcher
from devtools_testutils.aio import recorded_by_proxy_async

from _async_test_case import KeyVaultBackupClientPreparer, KeyVaultBackupClientSasPreparer
from _test_case import get_decorator
from _shared.test_case_async import KeyVaultTestCase

all_api_versions = get_decorator(is_async=True)
only_default = get_decorator(is_async=True, api_versions=[DEFAULT_VERSION])


class TestBackupClientTests(KeyVaultTestCase):
    def create_key_client(self, vault_uri, **kwargs):
        from azure.keyvault.keys.aio import KeyClient

        credential = self.get_credential(KeyClient, is_async=True)
        return self.create_client_from_credential(KeyClient, credential=credential, vault_url=vault_uri, **kwargs)

    @pytest.mark.asyncio
    @pytest.mark.parametrize("api_version", only_default)
    @KeyVaultBackupClientPreparer()
    @recorded_by_proxy_async
    async def test_full_backup_and_restore(self, client, **kwargs):
        set_bodiless_matcher()
        # backup the vault
        container_uri = kwargs.pop("container_uri")
        # make sure an error isn't raised by pre-backup check; i.e. ensure the backup can be done
        check_poller = await client.begin_pre_backup(container_uri, use_managed_identity=True)
        await check_poller.wait()
        backup_poller = await client.begin_backup(container_uri, use_managed_identity=True)
        backup_operation = await backup_poller.result()
        assert backup_operation.folder_url

        if self.is_live:
            await asyncio.sleep(15)  # Additional waiting to ensure backup will be available for restore

        # restore the backup
        # make sure an error isn't raised by pre-restore check; i.e. ensure the restore can be done
        check_poller = await client.begin_pre_restore(backup_operation.folder_url, use_managed_identity=True)
        await check_poller.wait()
        restore_poller = await client.begin_restore(backup_operation.folder_url, use_managed_identity=True)
        await restore_poller.wait()
        if self.is_live:
            await asyncio.sleep(60)  # additional waiting to avoid conflicts with resources in other tests

    @pytest.mark.asyncio
    @pytest.mark.parametrize("api_version", only_default)
    @KeyVaultBackupClientPreparer()
    @recorded_by_proxy_async
    async def test_full_backup_and_restore_rehydration(self, client, **kwargs):
        set_bodiless_matcher()

        # backup the vault
        container_uri = kwargs.pop("container_uri")
        backup_poller = await client.begin_backup(blob_storage_url=container_uri, use_managed_identity=True)

        # create a new poller from a continuation token
        token = backup_poller.continuation_token()
        rehydrated = await client.begin_backup(container_uri, use_managed_identity=True, continuation_token=token)

        if self.is_live:
            await asyncio.sleep(15)  # Additional waiting to ensure backup will be available for restore

        rehydrated_operation = await rehydrated.result()
        assert rehydrated_operation.folder_url
        backup_operation = await backup_poller.result()
        assert backup_operation.folder_url == rehydrated_operation.folder_url

        # restore the backup
        restore_poller = await client.begin_restore(backup_operation.folder_url, use_managed_identity=True)

        # create a new poller from a continuation token
        token = restore_poller.continuation_token()
        rehydrated = await client.begin_restore(
            backup_operation.folder_url, use_managed_identity=True, continuation_token=token
        )

        await rehydrated.wait()
        await restore_poller.wait()
        if self.is_live:
            await asyncio.sleep(60)  # additional waiting to avoid conflicts with resources in other tests

    @pytest.mark.asyncio
    @pytest.mark.parametrize("api_version", only_default)
    @KeyVaultBackupClientPreparer()
    @recorded_by_proxy_async
    async def test_selective_key_restore(self, client, **kwargs):
        # create a key to selectively restore
        managed_hsm_url = kwargs.pop("managed_hsm_url")
        key_client = self.create_key_client(managed_hsm_url, is_async=True)
        key_name = self.get_resource_name("selective-restore-test-key")
        await key_client.create_rsa_key(key_name)

        # backup the vault
        container_uri = kwargs.pop("container_uri")
        backup_poller = await client.begin_backup(container_uri, use_managed_identity=True)
        backup_operation = await backup_poller.result()

        if self.is_live:
            await asyncio.sleep(15)  # Additional waiting to ensure backup will be available for restore

        # restore the key
        restore_poller = await client.begin_restore(
            backup_operation.folder_url, use_managed_identity=True, key_name=key_name
        )
        await restore_poller.wait()

        # delete the key
        await self._poll_until_no_exception(key_client.delete_key, key_name, expected_exception=ResourceExistsError)
        await key_client.purge_deleted_key(key_name)
        if self.is_live:
            await asyncio.sleep(60)  # additional waiting to avoid conflicts with resources in other tests

    @pytest.mark.asyncio
    @pytest.mark.parametrize("api_version", only_default)
    @KeyVaultBackupClientPreparer()
    @recorded_by_proxy_async
    async def test_backup_client_polling(self, client, **kwargs):
        set_bodiless_matcher()

        # backup the vault
        container_uri = kwargs.pop("container_uri")
        backup_poller = await client.begin_backup(container_uri, use_managed_identity=True)

        # create a new poller from a continuation token
        token = backup_poller.continuation_token()
        rehydrated = await client.begin_backup(container_uri, use_managed_identity=True, continuation_token=token)

        # check that pollers and polling methods behave as expected
        if self.is_live:
            assert backup_poller.status() == "InProgress"
        assert not backup_poller.done() or backup_poller.polling_method().finished()
        # assert rehydrated.status() == "InProgress"
        assert not rehydrated.done() or rehydrated.polling_method().finished()

        backup_operation = await backup_poller.result()
        assert backup_poller.status() == "Succeeded" and backup_poller.polling_method().status() == "Succeeded"
        rehydrated_operation = await rehydrated.result()
        assert rehydrated.status() == "Succeeded" and rehydrated.polling_method().status() == "Succeeded"
        assert backup_operation.folder_url == rehydrated_operation.folder_url

        # rehydrate a poller with a continuation token of a completed operation
        late_rehydrated = await client.begin_backup(container_uri, use_managed_identity=True, continuation_token=token)
        assert late_rehydrated.status() == "Succeeded"
        await late_rehydrated.wait()

        if self.is_live:
            await asyncio.sleep(15)  # Additional waiting to ensure backup will be available for restore

        # restore the backup
        restore_poller = await client.begin_restore(backup_operation.folder_url, use_managed_identity=True)

        # create a new poller from a continuation token
        token = restore_poller.continuation_token()
        rehydrated = await client.begin_restore(
            backup_operation.folder_url, use_managed_identity=True, continuation_token=token
        )

        # check that pollers and polling methods behave as expected
        if self.is_live:
            assert restore_poller.status() == "InProgress"
        assert not restore_poller.done() or restore_poller.polling_method().finished()
        # assert rehydrated.status() == "InProgress"
        assert not rehydrated.done() or rehydrated.polling_method().finished()

        await rehydrated.wait()
        assert rehydrated.status() == "Succeeded" and rehydrated.polling_method().status() == "Succeeded"
        await restore_poller.wait()
        assert restore_poller.status() == "Succeeded" and restore_poller.polling_method().status() == "Succeeded"

        if self.is_live:
            await asyncio.sleep(60)  # additional waiting to avoid conflicts with resources in other tests

    @pytest.mark.live_test_only
    @pytest.mark.asyncio
    @pytest.mark.parametrize("api_version", only_default)
    @KeyVaultBackupClientSasPreparer()
    async def test_backup_restore_sas(self, client, **kwargs):
        # backup the vault
        container_uri = kwargs.pop("container_uri")
        sas_token = kwargs.pop("sas_token")

        if self.is_live and not sas_token:
            pytest.skip(
                "SAS token is required for live tests. Please set the BLOB_STORAGE_SAS_TOKEN environment variable."
            )

        check_poller = await client.begin_pre_backup(container_uri, sas_token=sas_token)
        await check_poller.wait()
        backup_poller = await client.begin_backup(container_uri, sas_token)
        backup_operation = await backup_poller.result()
        assert backup_operation.folder_url

        if self.is_live:
            await asyncio.sleep(15)  # Additional waiting to ensure backup will be available for restore

        # restore the backup
        check_poller = await client.begin_pre_restore(backup_operation.folder_url, sas_token=sas_token)
        check_result = await check_poller.result()
        assert check_result.error is None
        restore_poller = await client.begin_restore(backup_operation.folder_url, sas_token)
        await restore_poller.wait()
        if self.is_live:
            await asyncio.sleep(60)  # additional waiting to avoid conflicts with resources in other tests



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
from enum import Enum

from azure.core import CaseInsensitiveEnumMeta


class KeyVaultRoleScope(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Collection of well known role scopes. This list is not exhaustive."""

    GLOBAL = "/"  #: use this if you want role assignments to apply to everything on the resource
    KEYS = "/keys"  #: use this if you want role assignments to apply to all keys


class KeyVaultDataAction(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Supported permissions for data actions."""

    #: Read HSM key metadata.
    READ_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/read/action"
    #: Update an HSM key.
    WRITE_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/write/action"
    #: Read deleted HSM key.
    READ_DELETED_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/deletedKeys/read/action"
    #: Recover deleted HSM key.
    RECOVER_DELETED_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/deletedKeys/recover/action"
    #: Backup HSM keys.
    BACKUP_HSM_KEYS = "Microsoft.KeyVault/managedHsm/keys/backup/action"
    #: Restore HSM keys.
    RESTORE_HSM_KEYS = "Microsoft.KeyVault/managedHsm/keys/restore/action"
    #: Delete role assignment.
    DELETE_ROLE_ASSIGNMENT = "Microsoft.KeyVault/managedHsm/roleAssignments/delete/action"
    #: Get role assignment.
    GET_ROLE_ASSIGNMENT = "Microsoft.KeyVault/managedHsm/roleAssignments/read/action"
    #: Create or update role assignment.
    WRITE_ROLE_ASSIGNMENT = "Microsoft.KeyVault/managedHsm/roleAssignments/write/action"
    #: Get role definition.
    READ_ROLE_DEFINITION = "Microsoft.KeyVault/managedHsm/roleDefinitions/read/action"
    #: Create or update role definition.
    WRITE_ROLE_DEFINITION = "Microsoft.KeyVault/managedHsm/roleDefinitions/write/action"
    #: Delete role definition.
    DELETE_ROLE_DEFINITION = "Microsoft.KeyVault/managedHsm/roleDefinitions/delete/action"
    #: Encrypt using an HSM key.
    ENCRYPT_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/encrypt/action"
    #: Decrypt using an HSM key.
    DECRYPT_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/decrypt/action"
    #: Wrap using an HSM key.
    WRAP_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/wrap/action"
    #: Unwrap using an HSM key.
    UNWRAP_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/unwrap/action"
    #: Sign using an HSM key.
    SIGN_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/sign/action"
    #: Verify using an HSM key.
    VERIFY_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/verify/action"
    #: Create an HSM key.
    CREATE_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/create"
    #: Delete an HSM key.
    DELETE_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/delete"
    #: Export an HSM key.
    EXPORT_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/export/action"
    #: Release an HSM key using Secure Key Release.
    RELEASE_KEY = "Microsoft.KeyVault/managedHsm/keys/release/action"
    #: Import an HSM key.
    IMPORT_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/import/action"
    #: Purge a deleted HSM key.
    PURGE_DELETED_HSM_KEY = "Microsoft.KeyVault/managedHsm/keys/deletedKeys/delete"
    #: Download an HSM security domain.
    DOWNLOAD_HSM_SECURITY_DOMAIN = "Microsoft.KeyVault/managedHsm/securitydomain/download/action"
    #: Check status of HSM security domain download.
    DOWNLOAD_HSM_SECURITY_DOMAIN_STATUS = "Microsoft.KeyVault/managedHsm/securitydomain/download/read"
    #: Upload an HSM security domain.
    UPLOAD_HSM_SECURITY_DOMAIN = "Microsoft.KeyVault/managedHsm/securitydomain/upload/action"
    #: Check the status of the HSM security domain exchange file.
    READ_HSM_SECURITY_DOMAIN_STATUS = "Microsoft.KeyVault/managedHsm/securitydomain/upload/read"
    #: Download an HSM security domain transfer key.
    READ_HSM_SECURITY_DOMAIN_TRANSFER_KEY = "Microsoft.KeyVault/managedHsm/securitydomain/transferkey/read"
    #: Start an HSM backup.
    START_HSM_BACKUP = "Microsoft.KeyVault/managedHsm/backup/start/action"
    #: Start an HSM restore.
    START_HSM_RESTORE = "Microsoft.KeyVault/managedHsm/restore/start/action"
    #: Read an HSM backup status.
    READ_HSM_BACKUP_STATUS = "Microsoft.KeyVault/managedHsm/backup/status/action"
    #: Read an HSM restore status.
    READ_HSM_RESTORE_STATUS = "Microsoft.KeyVault/managedHsm/restore/status/action"
    #: Generate random numbers.
    RANDOM_NUMBERS_GENERATE = "Microsoft.KeyVault/managedHsm/rng/action"


class KeyVaultSettingType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type specifier of the setting value."""

    BOOLEAN = "boolean"



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from copy import deepcopy
import sys
from typing import Any, TYPE_CHECKING

from azure.core import PipelineClient
from azure.core.pipeline import policies
from azure.core.rest import HttpRequest, HttpResponse

from ._configuration import KeyVaultClientConfiguration
from ._utils.serialization import Deserializer, Serializer
from .operations import RoleAssignmentsOperations, RoleDefinitionsOperations, _KeyVaultClientOperationsMixin

if sys.version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self  # type: ignore

if TYPE_CHECKING:
    from azure.core.credentials import TokenCredential


class KeyVaultClient(_KeyVaultClientOperationsMixin):
    """The Azure Key Vault Administration service client performs administrative operations including
    RBAC, BackupRestore, and settings management against the Azure Key Vault service.

    :ivar role_definitions: RoleDefinitionsOperations operations
    :vartype role_definitions:
     azure.keyvault.administration._generated.operations.RoleDefinitionsOperations
    :ivar role_assignments: RoleAssignmentsOperations operations
    :vartype role_assignments:
     azure.keyvault.administration._generated.operations.RoleAssignmentsOperations
    :param vault_base_url: The base URL of the Key Vault instance (e.g.
     `https://myvault.vault.azure.net/ <https://myvault.vault.azure.net/>`_). Required.
    :type vault_base_url: str
    :param credential: Credential used to authenticate requests to the service. Required.
    :type credential: ~azure.core.credentials.TokenCredential
    :keyword api_version: The API version to use for this operation. Known values are
     "2026-01-01-preview" and None. Default value is None. If not set, the operation's default API
     version will be used. Note that overriding this default value may result in unsupported
     behavior.
    :paramtype api_version: str
    :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
     Retry-After header is present.
    """

    def __init__(self, vault_base_url: str, credential: "TokenCredential", **kwargs: Any) -> None:
        _endpoint = "{vaultBaseUrl}"
        self._config = KeyVaultClientConfiguration(vault_base_url=vault_base_url, credential=credential, **kwargs)

        _policies = kwargs.pop("policies", None)
        if _policies is None:
            _policies = [
                policies.RequestIdPolicy(**kwargs),
                self._config.headers_policy,
                self._config.user_agent_policy,
                self._config.proxy_policy,
                policies.ContentDecodePolicy(**kwargs),
                self._config.redirect_policy,
                self._config.retry_policy,
                self._config.authentication_policy,
                self._config.custom_hook_policy,
                self._config.logging_policy,
                policies.DistributedTracingPolicy(**kwargs),
                policies.SensitiveHeaderCleanupPolicy(**kwargs) if self._config.redirect_policy else None,
                self._config.http_logging_policy,
            ]
        self._client: PipelineClient = PipelineClient(base_url=_endpoint, policies=_policies, **kwargs)

        self._serialize = Serializer()
        self._deserialize = Deserializer()
        self._serialize.client_side_validation = False
        self.role_definitions = RoleDefinitionsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.role_assignments = RoleAssignmentsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )

    def send_request(self, request: HttpRequest, *, stream: bool = False, **kwargs: Any) -> HttpResponse:
        """Runs the network request through the client's chained policies.

        >>> from azure.core.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = client.send_request(request)
        <HttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~azure.core.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~azure.core.rest.HttpResponse
        """

        request_copy = deepcopy(request)
        path_format_arguments = {
            "vaultBaseUrl": self._serialize.url(
                "self._config.vault_base_url", self._config.vault_base_url, "str", skip_quote=True
            ),
        }

        request_copy.url = self._client.format_url(request_copy.url, **path_format_arguments)
        return self._client.send_request(request_copy, stream=stream, **kwargs)  # type: ignore

    def close(self) -> None:
        self._client.close()

    def __enter__(self) -> Self:
        self._client.__enter__()
        return self

    def __exit__(self, *exc_details: Any) -> None:
        self._client.__exit__(*exc_details)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models import (  # type: ignore
    EkmConnection,
    EkmProxyClientCertificateInfo,
    EkmProxyInfo,
    FullBackupOperation,
    FullBackupOperationError,
    KeyVaultError,
    Permission,
    PreBackupOperationParameters,
    PreRestoreOperationParameters,
    RestoreOperation,
    RestoreOperationParameters,
    RoleAssignment,
    RoleAssignmentCreateParameters,
    RoleAssignmentProperties,
    RoleAssignmentPropertiesWithScope,
    RoleDefinition,
    RoleDefinitionCreateParameters,
    RoleDefinitionProperties,
    SASTokenParameter,
    SelectiveKeyRestoreOperation,
    SelectiveKeyRestoreOperationParameters,
    Setting,
    SettingsListResult,
    UpdateSettingRequest,
)

from ._enums import (  # type: ignore
    DataAction,
    OperationStatus,
    RoleDefinitionType,
    RoleScope,
    RoleType,
    SettingTypeEnum,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "EkmConnection",
    "EkmProxyClientCertificateInfo",
    "EkmProxyInfo",
    "FullBackupOperation",
    "FullBackupOperationError",
    "KeyVaultError",
    "Permission",
    "PreBackupOperationParameters",
    "PreRestoreOperationParameters",
    "RestoreOperation",
    "RestoreOperationParameters",
    "RoleAssignment",
    "RoleAssignmentCreateParameters",
    "RoleAssignmentProperties",
    "RoleAssignmentPropertiesWithScope",
    "RoleDefinition",
    "RoleDefinitionCreateParameters",
    "RoleDefinitionProperties",
    "SASTokenParameter",
    "SelectiveKeyRestoreOperation",
    "SelectiveKeyRestoreOperationParameters",
    "Setting",
    "SettingsListResult",
    "UpdateSettingRequest",
    "DataAction",
    "OperationStatus",
    "RoleDefinitionType",
    "RoleScope",
    "RoleType",
    "SettingTypeEnum",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
import pytest

from azure.keyvault.administration import KeyVaultSetting, KeyVaultSettingsClient, KeyVaultSettingType
from azure.keyvault.administration._internal.client_base import DEFAULT_VERSION

from devtools_testutils import recorded_by_proxy

from _shared.test_case import KeyVaultTestCase
from _test_case import KeyVaultSettingsClientPreparer, get_decorator

only_latest = get_decorator(api_versions=[DEFAULT_VERSION])


class TestSettings(KeyVaultTestCase):
    @pytest.mark.parametrize("api_version", only_latest)
    @KeyVaultSettingsClientPreparer()
    @recorded_by_proxy
    def test_list_settings(self, client: KeyVaultSettingsClient, **kwargs):
        default_settings = [setting for setting in client.list_settings()]
        assert len(default_settings)
        for setting in default_settings:
            assert setting.name and setting.setting_type and setting.value is not None

    @pytest.mark.parametrize("api_version", only_latest)
    @KeyVaultSettingsClientPreparer()
    @recorded_by_proxy
    def test_update_settings(self, client: KeyVaultSettingsClient, **kwargs):
        setting = client.get_setting("AllowKeyManagementOperationsThroughARM")
        assert setting.name and setting.setting_type and setting.value

        # Set value by using a bool
        opposite_value = KeyVaultSetting(
            name=setting.name, value=not setting.getboolean(), setting_type=KeyVaultSettingType.BOOLEAN
        )
        updated = client.update_setting(opposite_value)
        assert updated.name == setting.name
        if setting.getboolean():
            assert not updated.getboolean()
        else:
            assert updated.getboolean()

        # Set value by using a string
        new_opposite = KeyVaultSetting(name=updated.name, value="false" if updated.getboolean() else "true")
        new_updated = client.update_setting(new_opposite)
        assert new_updated.name == updated.name
        assert new_updated.value != updated.value



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
import os

from azure.keyvault.administration import KeyVaultBackupResult

# ----------------------------------------------------------------------------------------------------------
# Prerequisites:
# 1. A managed HSM (https://learn.microsoft.com/azure/key-vault/managed-hsm/quick-create-cli)
#
# 2. azure-keyvault-administration and azure-identity libraries (pip install these)
#
# 3. Set environment variable MANAGED_HSM_URL with the URL of your managed HSM
#
# 4. A user-assigned managed identity that has access to your managed HSM. For more information about how to create a
#    user-assigned managed identity, refer to
#    https://learn.microsoft.com/entra/identity/managed-identities-azure-resources/overview
#
# 5. A storage account, that your managed identity has access to, containing a blob storage container
#    (See https://learn.microsoft.com/azure/storage/blobs/storage-blobs-introduction)
#
# 6. Set environment variables CONTAINER_URL and CLIENT_ID, corresponding to your blob container's URI and your
#    user-assigned managed identity's client ID, respectively.
#
# For more details, see https://learn.microsoft.com/azure/key-vault/managed-hsm/backup-restore
#
# ----------------------------------------------------------------------------------------------------------
# Sample - demonstrates full backup and restore operations for Managed HSM
#
# 1. Perform a full backup (begin_backup)
#
# 2. Perform a full restore (begin_restore)
# ----------------------------------------------------------------------------------------------------------

# Instantiate a backup client that will be used to call the service.
# Here we use the ManagedIdentityCredential to use Managed Identity, but any azure-identity credential can be used if
# opting for SAS token authentication.
# [START create_a_backup_restore_client]
from azure.identity import ManagedIdentityCredential
from azure.keyvault.administration import KeyVaultBackupClient

MANAGED_HSM_URL = os.environ["MANAGED_HSM_URL"]
MANAGED_IDENTITY_CLIENT_ID = os.environ["CLIENT_ID"]
credential = ManagedIdentityCredential(client_id=MANAGED_IDENTITY_CLIENT_ID)
client = KeyVaultBackupClient(vault_url=MANAGED_HSM_URL, credential=credential)
# [END create_a_backup_restore_client]

# Let's back up the vault with begin_backup, which returns a poller. Calling result() on the poller will return a
# KeyVaultBackupResult that contains the URL of the backup after the operation completes. Calling wait() on the
# poller will wait until the operation is complete.
print("\n.. Back up the vault")
# [START begin_backup]
CONTAINER_URL = os.environ["CONTAINER_URL"]

backup_result: KeyVaultBackupResult = client.begin_backup(CONTAINER_URL, use_managed_identity=True).result()
print(f"Azure Storage Blob URL of the backup: {backup_result.folder_url}")
# [END begin_backup]
print("Vault backed up successfully.")
assert backup_result.folder_url

# Now let's the vault by calling begin_restore, which also returns a poller. Calling result() on the poller will
# return None after the operation completes. Calling wait() on the poller will wait until the operation is complete.
# To restore a single key from the backed up vault instead, pass the key_name keyword argument.
print("\n.. Restore the full vault")
# [START begin_restore]
# `backup_result` is the KeyVaultBackupResult returned by `begin_backup`
client.begin_restore(backup_result.folder_url, use_managed_identity=True).wait()
print("Vault restored successfully.")
# [END begin_restore]
